/* Nokia 5110 LCD AVR Library
 *
 * Copyright (C) 2015 Sergey Denisov.
 * Written by Sergey Denisov aka LittleBuster (DenisovS21@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public Licence
 * as published by the Free Software Foundation; either version 3
 * of the Licence, or (at your option) any later version.
 *
 * Original library written by SkewPL, http://skew.tk
 */



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>


#include "nokia5110.h"

#include <avr/pgmspace.h>
#include <avr/io.h>
#include <util/delay.h>
#include "nokia5110_chars.h"


static struct {
    /* screen byte massive */
    uint8_t screen[504];

    /* cursor position */
    uint8_t cursor_x;
    uint8_t cursor_y;

} nokia_lcd;

/**
 * Sending data to LCD
 * @bytes: data
 * @is_data: transfer mode: 1 - data; 0 - command;
 */
static void write(uint8_t bytes, uint8_t is_data)
{
	register uint8_t i;
	/* Enable controller */
	PORT_LCD &= ~(1 << LCD_SCE);

	/* We are sending data */
	if (is_data)
		PORT_LCD |= (1 << LCD_DC);
	/* We are sending commands */
	else
		PORT_LCD &= ~(1 << LCD_DC);

	/* Send bytes */
	for (i = 0; i < 8; i++) {
		/* Set data pin to byte state */
		if ((bytes >> (7-i)) & 0x01)
			PORT_LCD |= (1 << LCD_DIN);
		else
			PORT_LCD &= ~(1 << LCD_DIN);

		/* Blink clock */
		PORT_LCD |= (1 << LCD_CLK);
		PORT_LCD &= ~(1 << LCD_CLK);
	}

	/* Disable controller */
	PORT_LCD |= (1 << LCD_SCE);
}

static void write_cmd(uint8_t cmd)
{
	write(cmd, 0);
}

static void write_data(uint8_t data)
{
	write(data, 1);
}

/*
 * Public functions
 */

void nokia_lcd_init(void)
{
	register unsigned i;
	/* Set pins as output */
	DDR_LCD |= (1 << LCD_SCE);
	DDR_LCD |= (1 << LCD_RST);
	DDR_LCD |= (1 << LCD_DC);
	DDR_LCD |= (1 << LCD_DIN);
	DDR_LCD |= (1 << LCD_CLK);

	/* Reset display */
	PORT_LCD |= (1 << LCD_RST);
	PORT_LCD |= (1 << LCD_SCE);
	_delay_ms(10);
	PORT_LCD &= ~(1 << LCD_RST);
	_delay_ms(70);
	PORT_LCD |= (1 << LCD_RST);

	/*
	 * Initialize display
	 */
	/* Enable controller */
	PORT_LCD &= ~(1 << LCD_SCE);
	/* -LCD Extended Commands mode- */
	write_cmd(0x21);
	/* LCD bias mode 1:48 */
	write_cmd(0x13);
	/* Set temperature coefficient */
	write_cmd(0x06);
	/* Default VOP (3.06 + 66 * 0.06 = 7V) */
	write_cmd(0xC2);
	/* Standard Commands mode, powered down */
	write_cmd(0x20);
	/* LCD in normal mode */
	write_cmd(0x09);

	/* Clear LCD RAM */
	write_cmd(0x80);
	write_cmd(LCD_CONTRAST);
	for (i = 0; i < 504; i++)
		write_data(0x00);

	/* Activate LCD */
	write_cmd(0x08);
	write_cmd(0x0C);
}

void nokia_lcd_clear(void)
{
	register unsigned i;
	/* Set column and row to 0 */
	write_cmd(0x80);
	write_cmd(0x40);
	/*Cursor too */
	nokia_lcd.cursor_x = 0;
	nokia_lcd.cursor_y = 0;
	/* Clear everything (504 bytes = 84cols * 48 rows / 8 bits) */
	for(i = 0;i < 504; i++)
		nokia_lcd.screen[i] = 0x00;
}

void nokia_lcd_power(uint8_t on)
{
	write_cmd(on ? 0x20 : 0x24);
}

void nokia_lcd_set_pixel(uint8_t x, uint8_t y, uint8_t value)
{
	uint8_t *byte = &nokia_lcd.screen[y/8*84+x];
	if (value)
		*byte |= (1 << (y % 8));
	else
		*byte &= ~(1 << (y %8 ));
}

void nokia_lcd_write_char(char code, uint8_t scale)
{
	register uint8_t x, y;

	for (x = 0; x < 5*scale; x++)
		for (y = 0; y < 7*scale; y++)
			if (pgm_read_byte(&CHARSET[code-32][x/scale]) & (1 << y/scale))
				nokia_lcd_set_pixel(nokia_lcd.cursor_x + x, nokia_lcd.cursor_y + y, 1);
			else
				nokia_lcd_set_pixel(nokia_lcd.cursor_x + x, nokia_lcd.cursor_y + y, 0);

	nokia_lcd.cursor_x += 5*scale + 1;
	if (nokia_lcd.cursor_x >= 84) {
		nokia_lcd.cursor_x = 0;
		nokia_lcd.cursor_y += 7*scale + 1;
	}
	if (nokia_lcd.cursor_y >= 48) {
		nokia_lcd.cursor_x = 0;
		nokia_lcd.cursor_y = 0;
	}
}

void nokia_lcd_write_string(const char *str, uint8_t scale)
{
	while(*str)
		nokia_lcd_write_char(*str++, scale);
}

void nokia_lcd_set_cursor(uint8_t x, uint8_t y)
{
	nokia_lcd.cursor_x = x;
	nokia_lcd.cursor_y = y;
}

void nokia_lcd_render(void)
{
	register unsigned i;
	/* Set column and row to 0 */
	write_cmd(0x80);
	write_cmd(0x40);

	/* Write screen to display */
	for (i = 0; i < 504; i++)
		write_data(nokia_lcd.screen[i]);
}

int p1_pos=20;
int p2_pos=20;


void start_screen(){
 
	nokia_lcd_clear();

	nokia_lcd_set_cursor(12, 0);
	nokia_lcd_write_string("Nastasiu", 1);

	nokia_lcd_set_cursor(15, 10);
	nokia_lcd_write_string("Ciprian", 1);
	
	
	nokia_lcd_set_cursor(15, 20);
	nokia_lcd_write_string("335CC", 1);
	
	nokia_lcd_set_cursor(10, 35);
	nokia_lcd_write_string("PONG GAME", 1);

	nokia_lcd_render();
	
}


void second_screen(){

	nokia_lcd_clear();
	
	
	nokia_lcd_set_cursor(12, 0);
	nokia_lcd_write_string("Press", 1);

	nokia_lcd_set_cursor(15, 10);
	nokia_lcd_write_string("any", 1);
	
	
	nokia_lcd_set_cursor(15, 20);
	nokia_lcd_write_string("button", 1);
	
	nokia_lcd_set_cursor(10, 30);
	nokia_lcd_write_string("to start !", 1);

	nokia_lcd_render();

}
void draw_field(){

	int i;
	for (i = 5 ; i < 77;i++){
			
			nokia_lcd_set_cursor(i,5);
			nokia_lcd_write_string("-", 1);
			nokia_lcd_set_cursor(i,40);
			nokia_lcd_write_string("-", 1);
			}

}


void draw_pads(){

	int j;
	for(j=1; j<10; j++){
			
			nokia_lcd_set_cursor(3,p1_pos);
			nokia_lcd_write_string("|",1);
			nokia_lcd_set_cursor(78,p2_pos);
			nokia_lcd_write_string("|",1);
		}


}

void draw_screen(){

	int min = 9, max = 36;

	if(p1_pos < min)
		p1_pos = min;
	if(p1_pos > max)
		p1_pos = max;
	
	if(p2_pos < min)
		p2_pos = min;
	if(p2_pos > max)
		p2_pos = max; 

	nokia_lcd_clear();
	
	draw_field();
	draw_pads();

	

	nokia_lcd_set_cursor(42, 20);
	nokia_lcd_write_string("o", 1);
	
	nokia_lcd_render();

}

void start_sound(void){
	
	TCCR1A |= (1 << COM1A1) | (1 << COM1B1) | (1 << WGM11);
	TCCR1B |= (1 << WGM13) | (1 << WGM12) | (1 << CS11) | (1 << CS10);
	OCR1A = 200;
	ICR1 = 500;
	DDRD|=(1<<PD5);
	

}
void stop_sound(void){

	DDRD&= ~(1<<PD5);
	
}


int main(){

	nokia_lcd_init();
	
	DDRB &= ~ ((1<<PB0) |  (1<<PB1) | (1<<PB2) | (1<<PB3));
	PORTB |= ((1<<PB0) | (1<<PB1) | (1<<PB2) | (1<<PB3));
	
	start_screen();
	
	_delay_ms(5000);
	
	second_screen();
	
	while(1){
		
	
	if((PINB & (1<< PB0)) == 0 )
		break;
	
	if((PINB & (1<< PB1)) == 0 )
		break;
	
	if((PINB & (1<< PB2)) == 0 )
		break;
	
	if((PINB & (1<< PB3)) == 0 )
		break;
	
	}
	
	start_sound();
	_delay_ms(500);
	stop_sound();
	
	while(1){
	
	draw_screen();
	
	if((PINB & (1<< PB0)) == 0 )
		p1_pos -= 1;
	
	if((PINB & (1<< PB1)) == 0 )
		p1_pos += 1;
	
	if((PINB & (1<< PB2)) == 0 )
		p2_pos += 1;
	
	if((PINB & (1<< PB3)) == 0 )
		p2_pos -= 1;
	
	
	_delay_ms(25);
	
	}
	

return 0;
}
